@@ -6,4 +6,11 @@ simage <- function(data, Var1=NULL, Var2=NULL, ...){
   Matrix::image(as(as(as( t(M2) ,  "dMatrix"), "generalMatrix"), "CsparseMatrix"), #as(t(M2), Class = "dgCMatrix"), 
                 xlab=name1, ylab=name2, 
                 colorkey=TRUE, ...)
-}  
\ No newline at end of file
+}  
+
+simage2 <- function(X, ...){
+  
+  Matrix::image(as(as(as( X ,  "dMatrix"), "generalMatrix"), "CsparseMatrix"), #as(t(M2), Class = "dgCMatrix"), 
+                # xlab=name1, ylab=name2, 
+                colorkey=TRUE, ...)
+}  
